/************************************/
/* Definition of the constants      */
/************************************/
 
/*--------------------*/
/* Software constants */
/*--------------------*/
 
#define VERSION_SOFT  "1.0"
 
/*------------------------------------------------*/
/* Constants used by the NADF record construction */
/*------------------------------------------------*/
 
#define MAX_NADF_ID   1700        /* maximum number of different NADF    */
                                  /* identifiers                         */
#define MAX_NADF_VAL  2000        /* maximum number of fields composing  */
                                  /* a NADF record                       */
#define MAX_SPLIT     100         /* maximum number of different NADF    */
                                  /* identifiers                         */
#define MAX_LENGTH    65535       /* maximum length for a NADF field     */
                                  /* (2 bytes)                           */
#define HEAP_SIZE     400         /* default size of the heap            */
 
#define SPLIT         1           /* Flag indicating that the splitting  */
                                  /* must be done in all cases           */
#define NO_SPLIT      2           /* Flag indicating that the splitting  */
                                  /* must be only done if the number of  */
                                  /* fields with this identifier is > 1  */
 
/*------------------------------------------------------------------------*/
/* Constants used by the high level I/O interface and by the logical data */
/* conversion file                                                        */
/*------------------------------------------------------------------------*/
 
#ifndef MAX_FILES
#define MAX_FILES     20          /* maximum number of open files         */
#endif
 
#define MAX_BAS       150         /* maximum number of basic information  */
                                  /* per logical data conversion file     */
 
#define NAME_LEN      16          /* maximum length of the set and list   */
                                  /* names                                */
 
#define UTILIZATION   1           /* flag indicating the 'utilization'    */
                                  /* status                               */
#define GENERATION    2           /* flag indicating the 'generation'     */
                                  /* status                               */
 
#define HASH_ENTRY    100         /* number of entries of a hashing table */
 
#define LIST          1           /* flag indicating the 'list' type */
#define SET           2           /* flag indicating the 'set'  type */
 
#define BAS_ID_T      1           /* type of the basic ID array    */
#define BAS_NAM_T     2           /* type of the basic NAME array  */
#define LIST_NAM_T    3           /* type of the list NAME array   */
#define LIST_NAMI_T   4           /* type of the list NAMEID array */
#define LIST_ID_T     5           /* type of the list ID array     */
#define SET_NAM_T     6           /* type of the set NAME array    */
#define SET_NAMI_T    7           /* type of the set NAMEID array  */
#define SET_ID_T      8           /* type of the set ID array      */
 
#define PACKED        1           /* 'PACKED' flag of structure */
#define NO_PACKED     2           /* 'NO_PACKED' flag of structure */
 
 
#define BUFF_IN       2*BUFSIZ    /* size of the input buffer without     */
                                  /* 'unget'                              */
#define BUFF_OUT      2*BUFSIZ    /* size of the output buffer            */
 
#define MIN_UNGET     10          /* number of successive garantueed      */
                                  /* 'unget()'                            */
 
#define FILE_IN       2           /* flag indicating the input status of  */
                                  /* the file                             */
#define FILE_OUT      1           /* flag indicating the output status of */
                                  /* the file                             */
 
#define PROT_BIT      384         /* protection mode of the files created */
                                  /* by 'creat_OUT'                       */
 
/*--------------------------------------------*/
/* Constants used by the adaptation interface */
/*--------------------------------------------*/
 
#define NADF_FILE     1           /* flag indicating that the NADF record */
                                  /* must be stored in a NADF file        */
 
/* Constants used by the parser and lexical analyser of audit log */
 
#define MAX_REC_TYP  101          /* maximum identifier of a record type */
 
#define MAX_STR      256          /* maximum number of characters that can */
                                  /* be read at once                       */
 
#ifdef  INVERSE_FORM
#define MAGIC_NUM    "AR32W\0"    /* magic number of the Sinix-Es log file */
                                  /* in inverted form                      */
#else
#define MAGIC_NUM    "AR32WR\0"   /* magic number of the Sinix-Es log file */
#endif
 
/* Constants used by the parser and lexical analyser of auditmap file */
 
/*------------------------------------------------*/
/* Constants used by the initialization interface */
/*------------------------------------------------*/
 
 
/* Constants used by the functionality F3 interface */
 
#define MAXLEN       256          /* maximum length of a filename */
 
#define CLASS_HASH   200          /* number of entries in the hash table */
                                  /* during the events class treatment   */
 
#define CLASS_SORT   100          /* number of entries in the sort table */
                                  /* during the events class treatment   */
 
#define MAX_EVT      256          /* maximum number of different event */
                                  /* names                             */
 
#define MAC_FILE     0            /* flag indicating that the 4 ltdb files */
                                  /* must be treated  (default value)      */
 
#define NO_MAC_FILE  1            /* flag indicating that the 4 ltdb files */
                                  /* must not be treated                   */
 
#define DATA_SUFF    ".data"      /* suffix of audit data description file */
#define CONV_SUFF    ".conv"      /* suffix of conversion description file */
#define TRANS_SUFF   ".trans"     /* suffix of transformation rule file */
#define LOGD_SUFF    ".ldcf"      /* suffix of logic data conversion file */
 
/* Constants used by the treatment of the LTDB files */
 
#define LID_REC_LEN  200          /* length of a 'lid' record */
#define LTF_REC_LEN  32           /* maximum length of a 'ltf' name */
#define MAXLID_LEN   16           /* maximum length of a 'lid' value */
 
 
/*------------------*/
/* global constants */
/*------------------*/
 
#ifndef NONE
#define NONE         0            /* null flag */
#endif
 
#ifndef TRUE
#define TRUE         1            /* true value */
#endif
 
#ifndef FALSE
#define FALSE        0            /* false flag */
#endif
 
/****************************************************/
/* List of constants define in the security package */
/****************************************************/
 
#define ADT_BAMSG       6     /* length of bad_auth message field */
#define ADT_CRONSZ      128   /* length of cron description field */
#define ADT_DATESZ      3     /* length of date field */
#define ADT_MAXPATHLEN  1009  /* maximum length for pathnames */
#define ADT_NODESZ      7     /* lenght of event log node name */
#define ADT_SEQSZ       3     /* length of the sequence number field */
#define ADT_TTYSZ       32    /* length of the real tty device field */
#define ADT_VERLEN      8     /* length of the audit version */
#define CAT_SIZE        32    /* number of categories long integers */
