/*
 :******************************************************************
 :* MODULE-DESCRIPTION
 :*
 :* TITLE         (/ On line translation of native files  /)
 :*
 :* NAME          fadapter.c
 :*
 :* COPYRIGHT     (C)   SIEMENS NIXDORF INFORMATIONSSYSTEME AG  1992
 :*                     ALL RIGHTS RESERVED
 :*
 :* MEMORY-CLASS  6
 :*
 :* PROG-CONVENTS (/ <internal conventions (instance, names,
 :*                  external data, ...). None -> empty text>     /)
 :*
 :* PROCEDURE     vopen
 :* REMARKS       (/ open a native file for on lin translation to NADF format \)
 :* ENTRY         EXTERNAL
 :*
 :* PROCEDURE     vread
 :* REMARKS       (/ reading & translation to NADF format of the next record \)
 :* ENTRY         EXTERNAL
 :*
 :* PROCEDURE     charger_type
 :* REMARKS       (/ append an audit data of type 'type' to the NADF record \)  
 :* ENTRY         EXTERNAL
 :*
 :* PROCEDURE     align
 :* REMARKS       (/ align an audit data value  \)  
 :* ENTRY         EXTERNAL
 :*
 :* PROCEDURE     class_event
 :* REMARKS       (/ append the event class audit data to the NADF record \)
 :* ENTRY         EXTERNAL
 :*
 :* PROCEDURE     vclose
 :* REMARKS       (/ close a native file \)  
 :* ENTRY         EXTERNAL
 :*
 :* PROCEDURE     readslow
 :* REMARKS       (/ read bytes from a file \)  
 :* ENTRY         EXTERNAL
 :*
 :* PROCEDURE     vlseek
 :* REMARKS       (/ seek the end of the native file \)  
 :* ENTRY         EXTERNAL
 :*
 :*-----------------------------------------------------------------
 :*
 :* VERSION       001
 :*   CRDATE      1994-05-09
 :*   AUTHOR      (/ Mounji Abdelaziz        , FUNDP /)
 :*   UPDATE      (/ original issue /)
 :*
 :* END-MODULE-DESCRIPTION NADF_file .
 :******************************************************************
*/
#ifdef BS2000
#include <sys.time.h>
#endif
#ifdef VMS
#include <time.h>
#else
#include <sys/time.h>
#endif
#include <stdio.h>
#include <sys/fcntl.h>     /* definition of system I/O constants */
#include <sys/types.h>
#include <sys/user.h>
#include "vfs.h"
#include "label.h"
#include "audit.h"


#include "rec_tab.h"
#include "event.type.tab"
#include "id_audit"
#include "SUN_error.h"


#define ALIGN_REC   4         /* alignment number */



typedef struct timeval  au_timeval;
typedef struct timezone  au_timezone;

typedef struct audit_record aud_rec;
typedef struct statfs db_stat;

int fdesc;                    /* the file descriptor of the native file */
char *buffAddr, *tempo;
char *RLEN_PTR;
int isOnline = 0;
int var_header = 0;

	

/*	vopen()
*
*	argv[0], ..., argv[argc-1] are string 
* 	argv[1] must be the name of the native SunOS file to be opened
*	the other arguments are not used
*	The effect is to open the file named argv[1]
* 	If the open succeeds, 0 is returned and fdesc is set to the file descrip
*	Otherwise, SUNBADOPEN is returned
*
*/

vopen(argc, argv)
int argc; 
char *argv[];
    {
    char *malloc();
    char *option, c;
    int  buffLen =4*BUFSIZ;   /* Length of the buffer */
    int i;
   
   printf("vopen(");
   for (i=0; i<argc; i++)
	printf("%s ", argv[i]);
   printf(")\n");

   RLEN_PTR = malloc(buffLen);
   buffAddr = malloc(buffLen);    /* Ask for a dynamic area */
   if (argc == 0) {		/* read from stdin off-line */
	fdesc = 0;
	return 0;
    }
    if ((*argv)[0] == '-') {
		option = *argv;
		while ((c = *++option) != '\0')  {
	    		switch (c) {
				case 'o':
			  		isOnline = 1;
					break;
				default :
			  		printf("vopen: illegal option: %c\n", c);
					exit(1);
	    		}
		}
		argv++;
		argc--;
    }
    if (argc == 1) { 		  
        	fdesc = open(*argv ,O_RDONLY);
        	if (fdesc >= 0)  { /* if the open succeeded */
	     		return(0);
    	        } else {
	     		free(RLEN_PTR);
	     		free(buffAddr);
	     		return SUNBADOPEN;
		}
    } else if (argc == 0) {  	
    		fdesc = 0;   /* read on-line from stdin */
    		return 0;
    } else {
		fprintf(stderr, "vopen: incorrect argument count\n");
		exit(1);
    }
	
}

/*	vread()
*
*	convert the next native record to NADF format and put this record 
* 	at the address pointed by *recAddr;
*	If the trailer record is encountered, the  field at_namelen of this
* 	of this trailer record is stored at the address nextPath;
* 	If the native record is successfully read and translated to NADF format
* 	0 is returned;
* 	otherwise if the trailer record is encountered, SUNTRAILER is returned
*	otherwise if an errr occurrs, an error code is returned  
* 
*/
	
vread(recAddr)
char **recAddr;
{
    int  nb_read=0, dbsz, temp, rc, i, rlen ; 
    int  *ih, *ent;
    int  charger_I(),charger_SH(),charger_STR(),charger_L(); 
    int  charger_db_L(), charger_db_I();
    char *nextPath;
    short rec_type;

    unsigned int *uent;
    short *sh,*sz; 
    long *lg; 
    char *pdf,*rlen_ptr, *pdf1, *ctime();
    aud_rec *ph;
    audit_trailer_t *atrail;
    char *pci, *pcl;
    db_stat  *pstatfs;
    au_timeval *ptimeval;
    au_timezone *ptimezone;
    audit_header_t *ahead;
    int j, len_header;
    long *lh;
    char * malloc();
    int fd;

    int  buffLen =4*BUFSIZ;   /* Length of the buffer */

        fd = fdesc;
	rlen_ptr = RLEN_PTR;
	tempo = rlen_ptr;
	if (var_header == 0) 
		{
	var_header = 1; 
	nb_read = readslow(fd, buffAddr, sizeof(audit_header_t));
 	if (nb_read  < sizeof(audit_header_t)) 
		if (nb_read == 0)            
                  	return SUNEOF;
        else
		  return SUNBADREAD;
	ahead=(audit_header_t *) buffAddr;
	len_header=ahead->ah_namelen;

	tempo=rlen_ptr;
	rlen=sizeof(int)+6*sizeof(short);
	rlen_ptr=rlen_ptr+sizeof(int);
	sh=(short *) rlen_ptr;

	rlen+=charger_I(&sh,magic_number,ahead->ah_magic);
	rlen+=charger_L(&sh,ahtime,ahead->ah_time);
	rlen+=charger_SH(&sh,namelen,ahead->ah_namelen);
	ih=(int *) tempo;
	*ih=rlen;

	nb_read= readslow(fd,buffAddr,len_header);
	if(nb_read < 0 || nb_read != len_header){
        	return SUNBADREAD;
	}

	*recAddr = tempo;   /* return the address of the header audit trail */
	return 0;
	}

        rlen_ptr=tempo;
        nb_read = readslow(fd, buffAddr, sizeof(audit_trailer_t));
        if (nb_read == 0)            
                  return SUNEOF;
	else if (nb_read < 0)
		  return SUNBADREAD;
        ph = (aud_rec *) buffAddr;
	rec_type = ph->au_record_type;
       if (rec_type >= AUR_MINRECTYPE && rec_type <= AUR_MAXRECTYPE)  {
		/* static int counter;

		counter++;
		printf("counter:%d\n", counter);
		if (counter == 3060) {
			counter = counter;
		}
		*/
		nb_read = readslow(fd, buffAddr+sizeof(audit_trailer_t),
				       sizeof(audit_record_t)-sizeof(audit_trailer_t));
        	if (nb_read == 0)            
                  	return SUNEOF;
		else if (nb_read < 0)
		  	return SUNBADREAD;

	        rlen=sizeof(int)+26*sizeof(short);
		rlen_ptr=rlen_ptr+sizeof(int);
		sh=(short *) rlen_ptr;
		rlen+=class_event(&sh,ph->au_event);
		rlen+=charger_L(&sh,time,ph->au_time);
		rlen+=charger_STR(&sh,record_type,
		record_type_tab[ph->au_record_type].record_name,13);
		rlen+=charger_SH(&sh,uid,ph->au_uid);
		rlen+=charger_SH(&sh,auid,ph->au_auid);
		rlen+=charger_SH(&sh,euid,ph->au_euid);
		rlen+=charger_USH(&sh,gid,ph->au_gid);
		rlen+=charger_SH(&sh,pid,ph->au_pid);
		rlen+=charger_I(&sh,errnb,ph->au_errno);
		rlen+=charger_I(&sh,return_value,ph->au_return);
		rlen+=charger_SH(&sh,label_level,ph->au_label.sl_level);	
		rlen+=charger_STR(&sh,label_categorie,
                                          ph->au_label.sl_categories,16);
		rlen+=charger_STR(&sh,label_unused,ph->au_label.sl_unused, 14);
		dbsz=ph->au_record_size-sizeof(aud_rec);
		temp=ph->au_param_count;
				
		nb_read=readslow(fd, buffAddr, dbsz);
        	if (nb_read < 0 || nb_read  < dbsz) 
		  	return SUNBADREAD;
	      
	switch (rec_type)
		 {
			case AUR_ACCESS:

			
			        sz=(short *) buffAddr;
			        pdf=buffAddr+sizeof(short)*temp;
				rlen=rlen+temp*2*sizeof(short);	
		 			
				rlen+=charger_db_S(&sh,vide_value,&pdf,*sz);	
			        pdf=pdf+*sz;
				sz++;
					 
				rlen+=charger_db_S(&sh,cur_root,&pdf,*sz);	
				pdf=pdf+*sz;
				sz++;

			       rlen+=charger_db_S(&sh,curw_directory,&pdf,*sz);
			       pdf=pdf+*sz;
			       sz++;
					 
			       rlen+=charger_db_S(&sh,file_name,&pdf,*sz);
			       ih=(int *) tempo;
			       *ih=rlen;
			       break;

			case AUR_ADJTIME:

				sz=(short *) buffAddr;
				pdf=buffAddr+sizeof(short)*temp;
				rlen=rlen+temp*2*sizeof(short);	

				rlen+=charger_db_S(&sh,vide_value,&pdf,*sz);	
				pdf=pdf+*sz;
				sz++;

				pcl = (char *) malloc(sizeof(long));
				copy_bytes(pdf, pcl, sizeof(long));
				lg=(long *) pcl;	 
				rlen+=charger_db_L(&sh,pdf,s_adj_by,*lg);
				free(pcl);	
				pdf=pdf+*sz;
				sz++;

				
				pcl = (char *) malloc(sizeof(long));
				copy_bytes(pdf, pcl, sizeof(long));
				lg=(long *) pcl;	 
				rlen+=charger_db_L(&sh,pdf,ms_adj_by,*lg);
				free(pcl);
				pdf=pdf+*sz;
				sz++;

				
				pcl = (char *) malloc(sizeof(long));
				copy_bytes(pdf, pcl, sizeof(long));
				lg=(long *) pcl;	 
				rlen+=charger_db_L(&sh,pdf,s_old,*lg);
				free(pcl);
				pdf=pdf+*sz;
				sz++;
				
				pcl = (char *) malloc(sizeof(long));
				copy_bytes(pdf, pcl, sizeof(long));
				lg=(long *) pcl;	 
				rlen+=charger_db_L(&sh,pdf,ms_old,*lg);
				ih=(int *) tempo;
				*ih=rlen;
				free(pcl);
				break;

			case AUR_CHMOD:
			
				sz=(short *) buffAddr;
				pdf=buffAddr+sizeof(short)*temp;
				rlen=rlen+temp*2*sizeof(short);	
					 
				rlen+=charger_db_S(&sh,vide_value,&pdf,*sz);
				pdf=pdf+*sz;
				sz++;
					  
				rlen+=charger_db_S(&sh,cur_root,&pdf,*sz);	
				pdf=pdf+*sz;
				sz++;
					 
			       rlen+=charger_db_S(&sh,curw_directory,&pdf,*sz);
				pdf=pdf+*sz;
				sz++;
					
				rlen+=charger_db_S(&sh,file_name,&pdf,*sz);
				pdf=pdf+*sz;
				sz++;

				pci=(char *) malloc(sizeof(int));
				copy_bytes(pdf,pci,sizeof(int));
				ent=(int *) pci;
				rlen+=charger_db_I(&sh,pdf,new_mode,*ent);
				ih=(int *) tempo;
				*ih=rlen;
				free(pci);
				break;

			case AUR_CHOWN:

			
						      
				sz=(short *) buffAddr;
				pdf=buffAddr+sizeof(short)*temp;
				rlen=rlen+temp*2*sizeof(short);	
					
				rlen+=charger_db_S(&sh,vide_value,&pdf,*sz);	
				pdf=pdf+*sz;
				sz++;
					 
				rlen+=charger_db_S(&sh,cur_root,&pdf,*sz);	
				pdf=pdf+*sz;
				sz++;
					 
			       rlen+=charger_db_S(&sh,curw_directory,&pdf,*sz);
				pdf=pdf+*sz;
				sz++;
					 
				rlen+=charger_db_S(&sh,file_name,&pdf,*sz);
				pdf=pdf+*sz;
				sz++;

				ent=(int *) pdf;
				rlen+=charger_I(&sh,new_usr,*ent);
				pdf=pdf+*sz;
				sz++;

				ent=(int *) pdf;
				rlen+=charger_I(&sh,new_group,*ent);
			
				ih=(int *) tempo;
				*ih=rlen;
                                break;

			case AUR_CHROOT:

			
				sz=(short *) buffAddr;
				pdf=buffAddr+sizeof(short)*temp;
				rlen=rlen+temp*2*sizeof(short);	
		
				rlen+=charger_db_S(&sh,vide_value,&pdf,*sz);	
				pdf=pdf+*sz;
				sz++;
					 
				rlen+=charger_db_S(&sh,cur_root,&pdf,*sz);	
				pdf=pdf+*sz;
				sz++;
				 
			       rlen+=charger_db_S(&sh,curw_directory,&pdf,*sz);
				pdf=pdf+*sz;
				sz++;
					 
				rlen+=charger_db_S(&sh,nr_name,&pdf,*sz);
				ih=(int *) tempo;
				*ih=rlen;
				break;

			case AUR_CHDIR:
				

									      
				sz=(short *) buffAddr;
				pdf=buffAddr+sizeof(short)*temp;
				rlen=rlen+temp*2*sizeof(short);	
	
				rlen+=charger_db_S(&sh,vide_value,&pdf,*sz);	
				pdf=pdf+*sz;
				sz++;
					 
				rlen+=charger_db_S(&sh,cur_root,&pdf,*sz);	
				pdf=pdf+*sz;
				sz++;
		 
			       rlen+=charger_db_S(&sh,curw_directory,&pdf,*sz);
				pdf=pdf+*sz;
				sz++;
					 
				rlen+=charger_db_S(&sh,new_dir,&pdf,*sz);
				ih=(int *) tempo;
				*ih=rlen;
                                break;

			case AUR_CORE:
				
				sz=(short *) buffAddr;
				pdf=buffAddr+sizeof(short)*temp;
				rlen=rlen+temp*2*sizeof(short);	
		 			
				rlen+=charger_db_S(&sh,vide_value,&pdf,*sz);	
				pdf=pdf+*sz;
				sz++;
					 
				rlen+=charger_db_S(&sh,cur_root,&pdf,*sz);	
				pdf=pdf+*sz;
				sz++;
			 
			       rlen+=charger_db_S(&sh,curw_directory,&pdf,*sz);
				pdf=pdf+*sz;
				sz++;
					 
				rlen+=charger_db_S(&sh,corefile,&pdf,*sz);
				ih=(int *) tempo;
				*ih=rlen;
				break;

			case AUR_CREAT:
				

			
				sz=(short *) buffAddr;
				pdf=buffAddr+sizeof(short)*temp;
				rlen=rlen+temp*2*sizeof(short);	
		 			
				rlen+=charger_db_S(&sh,vide_value,&pdf,*sz);	
				pdf=pdf+*sz;
				sz++;
		 
				rlen+=charger_db_S(&sh,cur_root,&pdf,*sz);	
				pdf=pdf+*sz;
				sz++;
			 
			       rlen+=charger_db_S(&sh,curw_directory,&pdf,*sz);
				pdf=pdf+*sz;
				sz++;
					 
				rlen+=charger_db_S(&sh,file_name,&pdf,*sz);
				ih=(int *) tempo;
				*ih=rlen;
				 break;

			case AUR_EXECV:
				

				sz=(short *) buffAddr;
				pdf=buffAddr+sizeof(short)*temp;
				rlen=rlen+temp*2*sizeof(short);	
		 			
				rlen+=charger_db_S(&sh,vide_value,&pdf,*sz);	
				pdf=pdf+*sz;
				sz++;
					 
				rlen+=charger_db_S(&sh,cur_root,&pdf,*sz);	
				pdf=pdf+*sz;
				sz++;
		 
			       rlen+=charger_db_S(&sh,curw_directory,&pdf,*sz);
				pdf=pdf+*sz;
				sz++;
					 
				rlen+=charger_db_S(&sh,path_name,&pdf,*sz);
				ih=(int *) tempo;
				*ih=rlen;
				break;

			case AUR_EXECVE:
				

				sz=(short *) buffAddr;
				pdf=buffAddr+sizeof(short)*temp;
				rlen=rlen+temp*2*sizeof(short);	
	
				rlen+=charger_db_S(&sh,vide_value,&pdf,*sz);	
				pdf=pdf+*sz;
				sz++;
					 
				rlen+=charger_db_S(&sh,cur_root,&pdf,*sz);	
				pdf=pdf+*sz;
				sz++;
					 
			       rlen+=charger_db_S(&sh,curw_directory,&pdf,*sz);
				pdf=pdf+*sz;
				sz++;
					 
				rlen+=charger_db_S(&sh,path_name,&pdf,*sz);
				ih=(int *) tempo;
				*ih=rlen;
				break;

			case AUR_FCHMOD:
				
						      
				sz=(short *) buffAddr;
				pdf=buffAddr+sizeof(short)*temp;
				rlen=rlen+temp*2*sizeof(short);	
					 
		  		rlen+=charger_db_S(&sh,vide_value,&pdf,*sz);
				pdf=pdf+*sz;
				sz++;

				pci = (char *) malloc(sizeof(int));
				copy_bytes(pdf, pci, sizeof(int));
				ent=(int *) pci;
			        rlen+= charger_db_I(&sh,pdf,file_descrp,*ent);
				free(pci);	
				pdf=pdf+*sz;
				sz++;

				pci = (char *) malloc(sizeof(int));
				copy_bytes(pdf, pci, sizeof(int));
				ent=(int *) pci;
			        rlen+= charger_db_I(&sh,pdf,new_mode,*ent);
				ih=(int *) tempo;
				*ih=rlen;
				free(pci);			
				break;

			case AUR_FCHOWN:
				

			 	sz=(short *) buffAddr;
				pdf=buffAddr+sizeof(short)*temp;
				rlen=rlen+temp*2*sizeof(short);	
					
				rlen+=charger_db_S(&sh,vide_value,&pdf,*sz);	
				pdf=pdf+*sz;
				sz++;
					 
				ent=(int *) pdf;
				rlen+=charger_I(&sh,file_descrp,*ent);	
				pdf=pdf+*sz;
				sz++;

				ent=(int *) pdf;
				rlen+=charger_I(&sh,new_usr,*ent);
				pdf=pdf+*sz;
				sz++;

				ent=(int *) pdf;
				rlen+=charger_I(&sh,new_group,*ent);
			
				ih=(int *) tempo;
				*ih=rlen;
                                break;

			case AUR_FTRUNCATE:
				
				sz=(short *) buffAddr;
				pdf=buffAddr+sizeof(short)*temp;
				rlen=rlen+temp*2*sizeof(short);	
					
				rlen+=charger_db_S(&sh,vide_value,&pdf,*sz);	
				pdf=pdf+*sz;
				sz++;

				pci = (char *) malloc(sizeof(int));
				pcl = (char *) malloc(sizeof(long));
				copy_bytes(pdf, pci, sizeof(int));
				ent=(int *) pci;
			        rlen+= charger_db_I(&sh,pdf,file_descrp,*ent);
				/* free(pci); Aziz bouble free Jan 13 1994*/ 
				pdf=pdf+*sz;
				sz++;

				copy_bytes(pdf, pcl, sizeof(long));
				lg=(long *) pcl;
			        rlen+= charger_db_L(&sh,pdf,l_trun_to,*lg);
				ih=(int *) tempo;
				*ih=rlen;
				free(pci);
				free(pcl);			
				break;

			case AUR_KILL:
				
				sz=(short *) buffAddr;
				pdf=buffAddr+sizeof(short)*temp;
				rlen=rlen+temp*2*sizeof(short);	
					
				rlen+=charger_db_S(&sh,vide_value,&pdf,*sz);	
				pdf=pdf+*sz;
				sz++;

				pci= (char *) malloc(2*sizeof(int));
				copy_bytes(pdf, pci, 2*sizeof(int));
				pdf=pci;
				ent=(int *) pdf;
				rlen+=charger_I(&sh,process_id,*ent);	
				pdf=pdf+*sz;
				sz++;

				ent=(int *) pdf;
				rlen+=charger_I(&sh,signal_nb,*ent);
				ih=(int *) tempo;
				*ih=rlen;
				free(pci);			
				break;

			case AUR_KILLPG:
				


				sz=(short *) buffAddr;
				pdf=buffAddr+sizeof(short)*temp;
				rlen=rlen+temp*2*sizeof(short);	
					
				rlen+=charger_db_S(&sh,vide_value,&pdf,*sz);	
				pdf=pdf+*sz;
				sz++;

				pci = (char *) malloc(sizeof(int));
				copy_bytes(pdf, pci, sizeof(int));
				ent=(int *) pci;
			      rlen+= charger_db_I(&sh,pdf,process_gr,*ent);
				free(pci);	
				pdf=pdf+*sz;
				sz++;

				pci = (char *) malloc(sizeof(int));
				copy_bytes(pdf, pci, sizeof(int));
				ent=(int *) pci;
			      rlen+= charger_db_I(&sh,pdf,signal_nb,*ent);
				ih=(int *) tempo;
				*ih=rlen;
				free(pci);			
				break;

			case AUR_LINK:
				

				sz=(short *) buffAddr;
			        pdf=buffAddr+sizeof(short)*temp;
				rlen=rlen+temp*2*sizeof(short);	
		 			
				rlen+=charger_db_S(&sh,vide_value,&pdf,*sz);	
				pdf=pdf+*sz;
				sz++;
				 
				rlen+=charger_db_S(&sh,cur_root,&pdf,*sz);	
				pdf=pdf+*sz;
				sz++;

					 
			       rlen+=charger_db_S(&sh,curw_directory,&pdf,*sz);
				pdf=pdf+*sz;
				sz++;
					 
				rlen+=charger_db_S(&sh,nf_link,&pdf,*sz);
				pdf=pdf+*sz;
				sz++;

				rlen+=charger_db_S(&sh,hd_linkn,&pdf,*sz);
				ih=(int *) tempo;
				*ih=rlen;
				break;


			case AUR_MKDIR:
				
				sz=(short *) buffAddr;
				pdf=buffAddr+sizeof(short)*temp;
				rlen=rlen+temp*2*sizeof(short);	
			
				rlen+=charger_db_S(&sh,vide_value,&pdf,*sz);	
				pdf=pdf+*sz;
				sz++;
					 
				rlen+=charger_db_S(&sh,cur_root,&pdf,*sz);	
				pdf=pdf+*sz;
				sz++;
			 
			       rlen+=charger_db_S(&sh,curw_directory,&pdf,*sz);
				pdf=pdf+*sz;
				sz++;
					 
				rlen+=charger_db_S(&sh,name_dir,&pdf,*sz);
				ih=(int *) tempo;
				*ih=rlen;
                                break;

			case AUR_MKNOD:

				sz=(short *) buffAddr;
				pdf=buffAddr+sizeof(short)*temp;
				rlen=rlen+temp*2*sizeof(short);	

				rlen+=charger_db_S(&sh,vide_value,&pdf,*sz);
				pdf=pdf+*sz;
				sz++;
					 
				rlen+=charger_db_S(&sh,cur_root,&pdf,*sz);
				pdf=pdf+*sz;
				sz++;
					 
			       rlen+=charger_db_S(&sh,curw_directory,&pdf,*sz);
				pdf=pdf+*sz;
				sz++;
					 
				rlen+=charger_db_S(&sh,file_name,&pdf,*sz);
				pdf=pdf+*sz;
				sz++;
					 
				rlen+=charger_db_S(&sh,mode,&pdf,*sz);
				ih=(int *) tempo;
				*ih=rlen;
				break;

			case AUR_MOUNT:
				
				sz=(short *) buffAddr;
				pdf=buffAddr+sizeof(short)*temp;
				rlen=rlen+temp*2*sizeof(short);	

		 			
				rlen+=charger_db_S(&sh,vide_value,&pdf,*sz);	
				pdf=pdf+*sz;
				sz++;

					 
				rlen+=charger_db_S(&sh,cur_root,&pdf,*sz);	
				pdf=pdf+*sz;
				sz++;

					 
			       rlen+=charger_db_S(&sh,curw_directory,&pdf,*sz);
				pdf=pdf+*sz;
				sz++;

				ent=(int *) pdf;	 
				rlen+=charger_I(&sh,type_mount,*ent);
				pdf=pdf+*sz;
				sz++;

			      rlen+=charger_db_S(&sh,loc_mount_dir,&pdf,*sz);
				pdf=pdf+*sz;
				sz++;

			 	ent=(int *) pdf;	 
				rlen+=charger_I(&sh,mount_flag,*ent);
				pdf=pdf+*sz;
				sz++;

				ent=(int *) pdf;	 
				rlen+=charger_I(&sh,mount_arg,*ent);
				ih=(int *) tempo;
				*ih=rlen;
				break;

			case AUR_MOUNT_UFS:
				

				sz=(short *) buffAddr;
				pdf=buffAddr+sizeof(short)*temp;
				rlen=rlen+temp*2*sizeof(short);	
		 			
				rlen+=charger_db_S(&sh,vide_value,&pdf,*sz);	
				pdf=pdf+*sz;
				sz++;
					 
				rlen+=charger_db_S(&sh,cur_root,&pdf,*sz);	
				pdf=pdf+*sz;
				sz++;
					 
			       rlen+=charger_db_S(&sh,curw_directory,&pdf,*sz);
				pdf=pdf+*sz;
				sz++;

				ent=(int *) pdf;	 
				rlen+=charger_I(&sh,type_mount,*ent);
				pdf=pdf+*sz;
				sz++;

			      rlen+=charger_db_S(&sh,loc_mount_dir,&pdf,*sz);
				pdf=pdf+*sz;
					  sz++;

			 	ent=(int *) pdf;	 
				rlen+=charger_I(&sh,mount_flag,*ent);
				pdf=pdf+*sz;
				sz++;

				ent=(int *) pdf;	 
				rlen+=charger_I(&sh,mount_block,*ent);
				ih=(int *) tempo;
				*ih=rlen;
				break;

			/* case AUR_MOUNT_NFS : ????????????????????*/

			case AUR_MSGCTL:
				

				sz=(short *) buffAddr;
				pdf=buffAddr+sizeof(short)*temp;
				rlen=rlen+temp*2*sizeof(short);	
		 			
				rlen+=charger_db_S(&sh,vide_value,&pdf,*sz);	
				pdf=pdf+*sz;
				sz++;

				ent=(int *) pdf;	 
				rlen+=charger_I(&sh,mesg_queue,*ent);
				ih=(int *) tempo;
				*ih=rlen;
				break;

			case AUR_MSGGET:
				
				sz=(short *) buffAddr;
					  pdf=buffAddr+sizeof(short)*temp;
					  rlen=rlen+temp*2*sizeof(short);	

		 			
				rlen+=charger_db_S(&sh,vide_value,&pdf,*sz);	
					  pdf=pdf+*sz;
					  sz++;

				pci = (char *) malloc(2*sizeof(int));
				copy_bytes(pdf,pci,2*sizeof(int));
				pdf=pci;
				ent=(int *) pdf;
				rlen+=charger_I(&sh,mesg_queue,*ent);
				pdf=pdf+*sz;
				sz++;

				ent=(int *) pdf;	 
				rlen+=charger_I(&sh,mesg_key,*ent);		
				ih=(int *) tempo;
				*ih=rlen;
				free(pci);
				break;

			case AUR_MSGRCV:
				

				sz=(short *) buffAddr;
				pdf=buffAddr+sizeof(short)*temp;
				rlen=rlen+temp*2*sizeof(short);	
		 			
				rlen+=charger_db_S(&sh,vide_value,&pdf,*sz);	
				pdf=pdf+*sz;
				sz++;

				pci = (char *) malloc(sizeof(int));
				copy_bytes(pdf, pci, sizeof(int));
				ent=(int *) pci;	 
				rlen+= charger_db_I(&sh,pdf,mesg_queue,*ent);
				ih=(int *) tempo;
				*ih=rlen;
				free(pci);
				break;

			case AUR_MSGSND:
				

				sz=(short *) buffAddr;
				pdf=buffAddr+sizeof(short)*temp;
				rlen=rlen+temp*2*sizeof(short);	
		 			
				rlen+=charger_db_S(&sh,vide_value,&pdf,*sz);	
				pdf=pdf+*sz;
				sz++;

				ent=(int *) pdf;	 
				rlen+=charger_I(&sh,mesg_queue,*ent);
				ih=(int *) tempo;
				*ih=rlen;
				break;

			case AUR_OPEN:
				

				sz=(short *) buffAddr;
				pdf=buffAddr+sizeof(short)*temp;
				rlen=rlen+temp*2*sizeof(short);	

				rlen+=charger_db_S(&sh,vide_value,&pdf,*sz);
				pdf=pdf+*sz;
				sz++;

					 
				rlen+=charger_db_S(&sh,cur_root,&pdf,*sz);
				pdf=pdf+*sz;
				sz++;
					 
			       rlen+=charger_db_S(&sh,curw_directory,&pdf,*sz);
				pdf=pdf+*sz;
				sz++;
					 
				rlen+=charger_db_S(&sh,file_name,&pdf,*sz);
				ih=(int *) tempo;
				*ih=rlen;
				break;

			case AUR_PTRACE:
				
						      
				sz=(short *) buffAddr;
				pdf=buffAddr+sizeof(short)*temp;
				rlen=rlen+temp*2*sizeof(short);	
		 			
				rlen+=charger_db_S(&sh,vide_value,&pdf,*sz);	
				pdf=pdf+*sz;
				sz++;

				ent=(int *) pdf;	 
				rlen+=charger_I(&sh,request,*ent);	
				pdf=pdf+*sz;
				sz++;

				ent=(int *) pdf;	 
			        rlen+=charger_I(&sh,proc_ptrace,*ent);
				pdf=pdf+*sz;
				sz++;

				ent=(int *) pdf;	 
				rlen+=charger_I(&sh,ptrace_1,*ent);
				pdf=pdf+*sz;
				sz++;

				ent=(int *) pdf;
				rlen+=charger_I(&sh,ptrace_2,*ent);
				pdf=pdf+*sz;
				sz++;

			 	ent=(int *) pdf;	 
				rlen+=charger_I(&sh,ptrace_3,*ent);		
				ih=(int *) tempo;
				*ih=rlen;
				break;

			case AUR_QUOTA_ON:
				
	      
				sz=(short *) buffAddr;
				pdf=buffAddr+sizeof(short)*temp;
				rlen=rlen+temp*2*sizeof(short);	
		
				rlen+=charger_db_S(&sh,vide_value,&pdf,*sz);	
				pdf=pdf+*sz;
				sz++;
			 
				rlen+=charger_db_S(&sh,cur_root,&pdf,*sz);	
				pdf=pdf+*sz;
				sz++;

				
			       rlen+=charger_db_S(&sh,curw_directory,&pdf,*sz);
				pdf=pdf+*sz;
				sz++;

				ent = (int *) pdf; 
				rlen+=charger_I(&sh,quota_cmd,*ent);
				pdf=pdf+*sz;
				sz++;

	 			rlen+=charger_db_S(&sh,device,&pdf,*sz);
				pdf=pdf+*sz;
				sz++;

				ent = (int *) pdf;
				rlen+=charger_I(&sh,quota,*ent);
				pdf=pdf+*sz;
				sz++;

			     rlen+=charger_db_S(&sh,quota_filename,&pdf,*sz);
				ih=(int *) tempo;
				*ih=rlen;
                                break;

			case AUR_QUOTA_OFF:
				
				sz=(short *) buffAddr;
				pdf=buffAddr+sizeof(short)*temp;
				rlen=rlen+temp*2*sizeof(short);	
		 			
				rlen+=charger_db_S(&sh,vide_value,&pdf,*sz);	
				pdf=pdf+*sz;
				sz++;

					 
				rlen+=charger_db_S(&sh,cur_root,&pdf,*sz);	
				pdf=pdf+*sz;
				sz++;

				
			       rlen+=charger_db_S(&sh,curw_directory,&pdf,*sz);
				pdf=pdf+*sz;
				sz++;

				ent = (int *) pdf; 
				rlen+=charger_I(&sh,quota_cmd,*ent);
				pdf=pdf+*sz;
				sz++;

	 			rlen+=charger_db_S(&sh,device,&pdf,*sz);
				pdf=pdf+*sz;
				sz++;

				ent = (int *) pdf;
				rlen+=charger_I(&sh,quota,*ent);
				ih=(int *) tempo;
				*ih=rlen;
                                break;

			case AUR_QUOTA_SET:
				

				sz=(short *) buffAddr;
				pdf=buffAddr+sizeof(short)*temp;
				rlen=rlen+temp*2*sizeof(short);	
		 			
				rlen+=charger_db_S(&sh,vide_value,&pdf,*sz);	
				pdf=pdf+*sz;
				sz++;
					 
				rlen+=charger_db_S(&sh,cur_root,&pdf,*sz);	
				pdf=pdf+*sz;
				sz++;

				
			       rlen+=charger_db_S(&sh,curw_directory,&pdf,*sz);
				pdf=pdf+*sz;
				sz++;

				ent = (int *) pdf; 
				rlen+=charger_I(&sh,quota_cmd,*ent);
				pdf=pdf+*sz;
				sz++;

	 			rlen+=charger_db_S(&sh,device,&pdf,*sz);
				pdf=pdf+*sz;
				sz++;

				ent = (int *) pdf;
				rlen+=charger_I(&sh,quota,*ent);
				pdf=pdf+*sz;
				sz++;

				uent=( unsigned int *) pdf;
				rlen+=charger_UI(&sh,dblk_1,*uent);
				pdf=pdf+*sz;
				sz++;

				uent=( unsigned int *) pdf;
				rlen+=charger_UI(&sh,dblk_2,*uent);
				pdf=pdf+*sz;
					  sz++;

				uent=( unsigned int *) pdf;
				rlen+=charger_UI(&sh,dblk_3,*uent);
				pdf=pdf+*sz;
				sz++;

				uent=( unsigned int *) pdf;
				rlen+=charger_UI(&sh,dblk_4,*uent);
				pdf=pdf+*sz;
				sz++;

				uent=( unsigned int *) pdf;
				rlen+=charger_UI(&sh,dblk_5,*uent);
				pdf=pdf+*sz;
					  sz++;

				uent=( unsigned int *) pdf;
				rlen+=charger_UI(&sh,dblk_6,*uent);
				pdf=pdf+*sz;
				sz++;

				uent=( unsigned int *) pdf;
				rlen+=charger_UI(&sh,dblk_7,*uent);
				pdf=pdf+*sz;
				sz++;

				uent=( unsigned int *) pdf;
				rlen+=charger_UI(&sh,dblk_8,*uent);	  
				ih=(int *) tempo;
				*ih=rlen;
                                break;

			case AUR_QUOTA_LIM:
				

				sz=(short *) buffAddr;
				pdf=buffAddr+sizeof(short)*temp;
				rlen=rlen+temp*2*sizeof(short);	
	 			
				rlen+=charger_db_S(&sh,vide_value,&pdf,*sz);	
				pdf=pdf+*sz;
				sz++;
					 
				rlen+=charger_db_S(&sh,cur_root,&pdf,*sz);	
				pdf=pdf+*sz;
				sz++;
				
			       rlen+=charger_db_S(&sh,curw_directory,&pdf,*sz);
				pdf=pdf+*sz;
				sz++;

				ent = (int *) pdf; 
				rlen+=charger_I(&sh,quota_cmd,*ent);
				pdf=pdf+*sz;
				sz++;

	 			rlen+=charger_db_S(&sh,device,&pdf,*sz);
				pdf=pdf+*sz;
				sz++;

				ent = (int *) pdf;
				rlen+=charger_I(&sh,quota,*ent);
				pdf=pdf+*sz;
				sz++;

				uent=( unsigned int *) pdf;
				rlen+=charger_UI(&sh,dblk_1,*uent);
				pdf=pdf+*sz;
				sz++;

				uent=( unsigned int *) pdf;
				rlen+=charger_UI(&sh,dblk_2,*uent);
				pdf=pdf+*sz;
				sz++;

				uent=( unsigned int *) pdf;
				rlen+=charger_UI(&sh,dblk_3,*uent);
				pdf=pdf+*sz;
				sz++;

				uent=( unsigned int *) pdf;
				rlen+=charger_UI(&sh,dblk_4,*uent);
				pdf=pdf+*sz;
				sz++;

				uent=( unsigned int *) pdf;
				rlen+=charger_UI(&sh,dblk_5,*uent);
				pdf=pdf+*sz;
				sz++;

				uent=( unsigned int *) pdf;
				rlen+=charger_UI(&sh,dblk_6,*uent);
				pdf=pdf+*sz;
				sz++;

				uent=( unsigned int *) pdf;
				rlen+=charger_UI(&sh,dblk_7,*uent);
				pdf=pdf+*sz;
			        sz++;

				uent=( unsigned int *) pdf;
				rlen+=charger_UI(&sh,dblk_8,*uent);	  
				ih=(int *) tempo;
				*ih=rlen;
                                break;

			case AUR_QUOTA_SYNC:
				
				sz=(short *) buffAddr;
				pdf=buffAddr+sizeof(short)*temp;
				rlen=rlen+temp*2*sizeof(short);	
			
				rlen+=charger_db_S(&sh,vide_value,&pdf,*sz);	
				pdf=pdf+*sz;
				sz++;

					 
				rlen+=charger_db_S(&sh,cur_root,&pdf,*sz);	
				pdf=pdf+*sz;
				sz++;

				
			       rlen+=charger_db_S(&sh,curw_directory,&pdf,*sz);
				pdf=pdf+*sz;
				sz++;

				ent = (int *) pdf; 
				rlen+=charger_I(&sh,quota_cmd,*ent);
				pdf=pdf+*sz;
				sz++;

	 			rlen+=charger_db_S(&sh,device,&pdf,*sz);
				pdf=pdf+*sz;
				sz++;

				ent = (int *) pdf;
				rlen+=charger_I(&sh,quota,*ent);
				ih=(int *) tempo;
			        *ih=rlen;
                                break;

			
			case AUR_QUOTA:
				
				sz=(short *) buffAddr;
				pdf=buffAddr+sizeof(short)*temp;
				rlen=rlen+temp*2*sizeof(short);	
		
				rlen+=charger_db_S(&sh,vide_value,&pdf,*sz);	
				pdf=pdf+*sz;
				sz++;
			 
				rlen+=charger_db_S(&sh,cur_root,&pdf,*sz);	
				pdf=pdf+*sz;
				sz++;
				
			     rlen+=charger_db_S(&sh,curw_directory,&pdf,*sz);
				pdf=pdf+*sz;
				sz++;

				ent = (int *) pdf; 
				rlen+=charger_I(&sh,quota_cmd,*ent);
				pdf=pdf+*sz;
				sz++;

	 			rlen+=charger_db_S(&sh,device,&pdf,*sz);
				pdf=pdf+*sz;
				sz++;

				ent = (int *) pdf;
				rlen+=charger_I(&sh,quota,*ent);
				pdf=pdf+*sz;
				sz++;

			        rlen+=charger_db_S(&sh,quota_arg,&pdf,*sz);
				ih=(int *) tempo;
				*ih=rlen;
                                break;

			case AUR_READLINK:
				
				sz=(short *) buffAddr;
				pdf=buffAddr+sizeof(short)*temp;
				rlen=rlen+temp*2*sizeof(short);	
		 			
				rlen+=charger_db_S(&sh,vide_value,&pdf,*sz);	
				pdf=pdf+*sz;
				sz++;
					 
				rlen+=charger_db_S(&sh,cur_root,&pdf,*sz);	
				pdf=pdf+*sz;
				sz++;
					 
			       rlen+=charger_db_S(&sh,curw_directory,&pdf,*sz);
			        pdf=pdf+*sz;
				sz++;
					 
				rlen+=charger_db_S(&sh,link_name,&pdf,*sz);
				pdf=pdf+*sz;
				sz++;
				
				rlen+=charger_db_S(&sh,buff_link,&pdf,*sz);
				pdf=pdf+*sz;
				sz++;

				pci = (char *) malloc(sizeof(int));
				copy_bytes(pdf, pci, sizeof(int));
				ent=(int *) pci;
			      rlen+= charger_db_I(&sh,pdf,count_buff,*ent);
				ih=(int *) tempo;
				*ih=rlen;
				free(pci);
				break;

			case AUR_REBOOT:
				
				sz=(short *) buffAddr;
				pdf=buffAddr+sizeof(short)*temp;
				rlen=rlen+temp*2*sizeof(short);	
		 			
				rlen+=charger_db_S(&sh,vide_value,&pdf,*sz);	
				pdf=pdf+*sz;
				sz++;

				ent=(int *) pdf;	 
				rlen+=charger_I(&sh,arg_reb,*ent);
				ih=(int *) tempo;
				*ih=rlen;
				break;
			

			case AUR_REBOOTFAIL:
				
				sz=(short *) buffAddr;
				pdf=buffAddr+sizeof(short)*temp;
			        rlen=rlen+temp*2*sizeof(short);	

		 			
				rlen+=charger_db_S(&sh,vide_value,&pdf,*sz);	
				pdf=pdf+*sz;
				sz++;

				ent=(int *) pdf;	 
				rlen+=charger_I(&sh,arg_reb,*ent);
				ih=(int *) tempo;
				*ih=rlen;
				break;

			case AUR_RENAME:
				

			

				sz=(short *) buffAddr;
				pdf=buffAddr+sizeof(short)*temp;
				rlen=rlen+temp*2*sizeof(short);	
		 			
				rlen+=charger_db_S(&sh,vide_value,&pdf,*sz);	
				pdf=pdf+*sz;
				sz++;
		 
				rlen+=charger_db_S(&sh,cur_root,&pdf,*sz);	
				pdf=pdf+*sz;
				sz++;
					 
			       rlen+=charger_db_S(&sh,curw_directory,&pdf,*sz);
				pdf=pdf+*sz;
				sz++;
					 
				rlen+=charger_db_S(&sh,from_fname,&pdf,*sz);
				pdf=pdf+*sz;
				sz++;

				rlen+=charger_db_S(&sh,to_fname,&pdf,*sz);
				ih=(int *) tempo;
				*ih=rlen;
                                break;


			case AUR_RMDIR:
				

				sz=(short *) buffAddr;
				pdf=buffAddr+sizeof(short)*temp;
			        rlen=rlen+temp*2*sizeof(short);	
		 			
				rlen+=charger_db_S(&sh,vide_value,&pdf,*sz);	
				pdf=pdf+*sz;
				sz++;
					 
				rlen+=charger_db_S(&sh,cur_root,&pdf,*sz);	
				pdf=pdf+*sz;
				sz++;
					 
			       rlen+=charger_db_S(&sh,curw_directory,&pdf,*sz);
				pdf=pdf+*sz;
				sz++;
					 
				rlen+=charger_db_S(&sh,name_dir,&pdf,*sz);
				ih=(int *) tempo;
				*ih=rlen;
                                break;


			case AUR_SEMCTL:
				

				sz=(short *) buffAddr;
				pdf=buffAddr+sizeof(short)*temp;
				rlen=rlen+temp*2*sizeof(short);	
 			
				rlen+=charger_db_S(&sh,vide_value,&pdf,*sz);	
				pdf=pdf+*sz;
				sz++;

				ent=(int *) pdf;	 
				rlen+=charger_I(&sh,sem_id,*ent);
				ih=(int *) tempo;
				*ih=rlen;
				break;			


			case AUR_SEMGET:
				
				sz=(short *) buffAddr;
				pdf=buffAddr+sizeof(short)*temp;
				rlen=rlen+temp*2*sizeof(short);	
			        rlen+=charger_db_S(&sh,vide_value,&pdf,*sz);	
				pdf=pdf+*sz;
				sz++;

				pci = (char *) malloc(sizeof(int));
				copy_bytes(pdf, pci, sizeof(int));
				ent=(int *) pci;
			        rlen+= charger_db_I(&sh,pdf,sem_id,*ent);	
				pdf=pdf+*sz;
				sz++;

				/* pci = (char *) malloc(sizeof(int)); Aziz */
				copy_bytes(pdf, pci, sizeof(int));
				ent=(int *) pci;
			        rlen+= charger_db_I(&sh,pdf,sem_key,*ent);
				ih=(int *) tempo;
				*ih=rlen;
				free(pci);
				break;

			case AUR_SEMOP:
				
				sz=(short *) buffAddr;
				pdf=buffAddr+sizeof(short)*temp;
				rlen=rlen+temp*2*sizeof(short);	
		
				rlen+=charger_db_S(&sh,vide_value,&pdf,*sz);	
				pdf=pdf+*sz;
				sz++;

				ent=(int *) pdf;	 
				rlen+=charger_I(&sh,sem_id,*ent);
				ih=(int *) tempo;
				*ih=rlen;
				break;

			case AUR_SETDOMAINNAME:
				
				sz=(short *) buffAddr;
				pdf=buffAddr+sizeof(short)*temp;
				rlen=rlen+temp*2*sizeof(short);	
		 			
				rlen+=charger_db_S(&sh,vide_value,&pdf,*sz);	
				pdf=pdf+*sz;
				sz++;
					 
				rlen+=charger_db_S(&sh,old_dname,&pdf,*sz);	
				pdf=pdf+*sz;
				sz++;
		 
			        rlen+=charger_db_S(&sh,new_dname,&pdf,*sz);  
				ih=(int *) tempo;
				*ih=rlen;
                                break;
			

			case AUR_SETHOSTNAME:
				
      
				sz=(short *) buffAddr;
				pdf=buffAddr+sizeof(short)*temp;
				rlen=rlen+temp*2*sizeof(short);	
	
				rlen+=charger_db_S(&sh,vide_value,&pdf,*sz);	
				pdf=pdf+*sz;
				sz++;
	 
				rlen+=charger_db_S(&sh,old_hname,&pdf,*sz);	
				pdf=pdf+*sz;
				sz++;
					 
			        rlen+=charger_db_S(&sh,new_hname,&pdf,*sz);	
				ih=(int *) tempo;
				*ih=rlen;
                                break;

			case AUR_SETTIMEOFDAY:
				
				sz=(short *) buffAddr;
				pdf=buffAddr+sizeof(short)*temp;
				rlen=rlen+temp*2*sizeof(short);	

				rlen+=charger_db_S(&sh,vide_value,&pdf,*sz);	
				pdf=pdf+*sz;
				sz++;

				pcl = (char *) malloc(2*sizeof(long));
				copy_bytes(pdf, pcl,2*sizeof(long));
				ptimeval=(au_timeval *) pcl;
				rlen+=charger_L(&sh,second,ptimeval->tv_sec);	
		        	rlen+=charger_L(&sh,msecond,ptimeval->tv_usec);
				pdf =pdf+*sz;

				pci = (char *) malloc(2*sizeof(int));
				copy_bytes(pdf, pci,2*sizeof(int));
				ptimezone=(au_timezone *) pci;	 
			rlen+=charger_I(&sh,minute,ptimezone->tz_minuteswest );
			rlen+=charger_I(&sh,type_dest,ptimezone->tz_dsttime );
				ih=(int *) tempo;
				*ih=rlen+8;
				free(pci);
				free(pcl);
				break;

			case AUR_SHMAT:
				

				sz=(short *) buffAddr;
				pdf=buffAddr+sizeof(short)*temp;
				rlen=rlen+temp*2*sizeof(short);	

				rlen+=charger_db_S(&sh,vide_value,&pdf,*sz);	
				pdf=pdf+*sz;
				sz++;

				ent=(int *) pdf;	 
				rlen+=charger_I(&sh,shm_id,*ent );
				ih=(int *) tempo;
				*ih=rlen;
				break;

			case AUR_SHMCTL:
				

				sz=(short *) buffAddr;
				pdf=buffAddr+sizeof(short)*temp;
				rlen=rlen+temp*2*sizeof(short);	

				rlen+=charger_db_S(&sh,vide_value,&pdf,*sz);	
				pdf=pdf+*sz;
				sz++;

				ent=(int *) pdf;	 
				rlen+=charger_I(&sh,shm_id,*ent );
				ih=(int *) tempo;
				*ih=rlen;
				break;

			case AUR_SHMDT:
				

				sz=(short *) buffAddr;
				pdf=buffAddr+sizeof(short)*temp;
				rlen=rlen+temp*2*sizeof(short);	

				rlen+=charger_db_S(&sh,vide_value,&pdf,*sz);	
				pdf=pdf+*sz;
				sz++;

				ent=(int *) pdf;	 
				rlen+=charger_I(&sh,shm_id,*ent );
				ih=(int *) tempo;
				*ih=rlen;
				break;

			case AUR_SHMGET:
				

				sz=(short *) buffAddr;
				pdf=buffAddr+sizeof(short)*temp;
				rlen=rlen+3*2*sizeof(short);	

				rlen+=charger_db_S(&sh,vide_value,&pdf,*sz);	
				pdf=pdf+*sz;
				sz++;

				ent=(int *) pdf;	 
				rlen+=charger_I(&sh,shm_seg,*ent );
				pdf=pdf+*sz;
				sz++;

				ent=(int *) pdf;	 
				rlen+=charger_I(&sh,shm_key,*ent );
				ih=(int *) tempo;
				*ih=rlen;
				break;

			case AUR_SOCKET:
				


				sz=(short *) buffAddr;
				pdf=buffAddr+sizeof(short)*temp;
				rlen=rlen+temp*2*sizeof(short);	

				rlen+=charger_db_S(&sh,vide_value,&pdf,*sz);	
				pdf=pdf+*sz;
				sz++;

				ent=(int *) pdf;	 
				rlen+=charger_I(&sh,adressf,*ent);	
				pdf=pdf+*sz;
				sz++;

				ent=(int *) pdf;	 
			        rlen+=charger_I(&sh,socket_type,*ent);
				pdf=pdf+*sz;
				sz++;

				ent=(int *) pdf;	 
				rlen+=charger_I(&sh,protocols,*ent );
				ih=(int *) tempo;
				*ih=rlen;
				break;		


			case AUR_STAT:
				

				sz=(short *) buffAddr;
				pdf=buffAddr+sizeof(short)*temp;
				rlen=rlen+temp*2*sizeof(short);	
		 			
				rlen+=charger_db_S(&sh,vide_value,&pdf,*sz);	
				pdf=pdf+*sz;
				sz++;
					 
				rlen+=charger_db_S(&sh,cur_root,&pdf,*sz);	
				pdf=pdf+*sz;
				sz++;
					 
	 		       rlen+=charger_db_S(&sh,curw_directory,&pdf,*sz);
				pdf=pdf+*sz;
				sz++;
					 
				rlen+=charger_db_S(&sh,file_name,&pdf,*sz);
				ih=(int *) tempo;
				*ih=rlen;				
				break;

			case AUR_STATFS:
				
				sz=(short *) buffAddr;
				pdf=buffAddr+sizeof(short)*temp;
				rlen=rlen+temp*2*sizeof(short);	

				rlen+=charger_db_S(&sh,vide_value,&pdf,*sz);	
				pdf=pdf+*sz;
				sz++;

				rlen+=charger_db_S(&sh,cur_root,&pdf,*sz);	
				pdf=pdf+*sz;
				sz++;

				rlen+=charger_db_S(&sh,curw_directory,&pdf,*sz);
				pdf=pdf+*sz;
				sz++;

				rlen+=charger_db_S(&sh,name_dir,&pdf,*sz);	
				pdf=pdf+*sz;
				sz++;
				

			        pcl = (char *) malloc(16*sizeof(long));
			        copy_bytes(pdf,pcl,16*sizeof(long));
				pstatfs = (db_stat *) pcl;
			        rlen+=charger_L(&sh,type_info,pstatfs->f_type);
				rlen+=charger_L(&sh,fsB_S,pstatfs->f_bsize);
			       rlen+=charger_L(&sh,totB_fs,pstatfs->f_blocks);
				rlen+=charger_L(&sh,free_Bfs,pstatfs->f_bfree);
			       rlen+=charger_L(&sh,free_Bnsu,pstatfs->f_bavail);
				rlen+=charger_L(&sh,totf_nfs,pstatfs->f_files);
				rlen+=charger_L(&sh,free_fnfs,pstatfs->f_ffree);
				
			  rlen+=charger_L(&sh,first_pfs,pstatfs->f_fsid.val[0]);
				
			 rlen+=charger_L(&sh,second_pfs,pstatfs->f_fsid.val[1]);
		               rlen+=charger_L(&sh,spare_1,pstatfs->f_spare[0]);
			       rlen+=charger_L(&sh,spare_2,pstatfs->f_spare[1]);
			       rlen+=charger_L(&sh,spare_3,pstatfs->f_spare[2]);
			       rlen+=charger_L(&sh,spare_4,pstatfs->f_spare[3]);
			       rlen+=charger_L(&sh,spare_5,pstatfs->f_spare[4]);
			       rlen+=charger_L(&sh,spare_6,pstatfs->f_spare[5]);
			       rlen+=charger_L(&sh,spare_7,pstatfs->f_spare[6]);
			       
				free(pcl);
				ih=(int *) tempo;
				*ih=rlen+60;
				break;

			case AUR_SYMLINK:
				
				sz=(short *) buffAddr;
				pdf=buffAddr+sizeof(short)*temp;
				rlen=rlen+temp*2*sizeof(short);	
					 
				rlen+=charger_db_S(&sh,vide_value,&pdf,*sz);
				pdf=pdf+*sz;
				sz++;
					  
				rlen+=charger_db_S(&sh,cur_root,&pdf,*sz);	
				pdf=pdf+*sz;
				sz++;
					 
			       rlen+=charger_db_S(&sh,curw_directory,&pdf,*sz);
				pdf=pdf+*sz;
				sz++;
					
				rlen+=charger_db_S(&sh,nf_link,&pdf,*sz);
				pdf=pdf+*sz;
				sz++;

				rlen+=charger_db_S(&sh,link_name,&pdf,*sz);
				ih=(int *) tempo;
				*ih=rlen;
				break;

			case AUR_SYSACCT:
				

				sz=(short *) buffAddr;
				pdf=buffAddr+sizeof(short)*temp;
				rlen=rlen+temp*2*sizeof(short);	
					 
				rlen+=charger_db_S(&sh,vide_value,&pdf,*sz);
				pdf=pdf+*sz;
				sz++;
					  
			     rlen+=charger_db_S(&sh,account_fname,&pdf,*sz);	
				ih=(int *) tempo;
				*ih=rlen;
				break;

			case AUR_TRUNCATE:
				

				sz=(short *) buffAddr;
				pdf=buffAddr+sizeof(short)*temp;
				rlen=rlen+temp*2*sizeof(short);	
					 
				rlen+=charger_db_S(&sh,vide_value,&pdf,*sz);
				pdf=pdf+*sz;
				sz++;
					  
				rlen+=charger_db_S(&sh,cur_root,&pdf,*sz);	
				pdf=pdf+*sz;
				sz++;
					 
			       rlen+=charger_db_S(&sh,curw_directory,&pdf,*sz);
				pdf=pdf+*sz;
				sz++;
					
				rlen+=charger_db_S(&sh,file_name,&pdf,*sz);
				pdf=pdf+*sz;
				sz++;

				pcl=(char *) malloc(sizeof(long));
				copy_bytes(pdf,pcl,sizeof(long));
				lg=(long *) pcl;
				rlen+=charger_db_L(&sh,pdf,l_trun_to,*lg);
				ih=(int *) tempo;
				*ih=rlen;
				free(pcl);
				break;

			case AUR_UNLINK:

				
			        sz=(short *) buffAddr;
			        pdf=buffAddr+sizeof(short)*temp;
				rlen=rlen+temp*2*sizeof(short);	
		 			
				rlen+=charger_db_S(&sh,vide_value,&pdf,*sz);	
			        pdf=pdf+*sz;
				sz++;
					 
				rlen+=charger_db_S(&sh,cur_root,&pdf,*sz);	
				pdf=pdf+*sz;
				sz++;

			       rlen+=charger_db_S(&sh,curw_directory,&pdf,*sz);
			       pdf=pdf+*sz;
			       sz++;
					 
			       rlen+=charger_db_S(&sh,file_name,&pdf,*sz);
			       ih=(int *) tempo;
			       *ih=rlen;
			       break;

			case AUR_UNMOUNT:

				

			
			        sz=(short *) buffAddr;
			        pdf=buffAddr+sizeof(short)*temp;
				rlen=rlen+temp*2*sizeof(short);	
		 			
				rlen+=charger_db_S(&sh,vide_value,&pdf,*sz);	
			        pdf=pdf+*sz;
				sz++;
					 
				rlen+=charger_db_S(&sh,cur_root,&pdf,*sz);	
				pdf=pdf+*sz;
				sz++;

			       rlen+=charger_db_S(&sh,curw_directory,&pdf,*sz);
			       pdf=pdf+*sz;
			       sz++;
					 
			      rlen+=charger_db_S(&sh,loc_mount_dir,&pdf,*sz);
			       ih=(int *) tempo;
			       *ih=rlen;
			       break;

			case AUR_UTIMES:

	     
			        sz=(short *) buffAddr;
			        pdf=buffAddr+sizeof(short)*temp;
				rlen=rlen+temp*2*sizeof(short);	
		 			
				rlen+=charger_db_S(&sh,vide_value,&pdf,*sz);	
			        pdf=pdf+*sz;
				sz++;
				
					 
				rlen+=charger_db_S(&sh,cur_root,&pdf,*sz);	
				pdf=pdf+*sz;
				sz++;

			       rlen+=charger_db_S(&sh,curw_directory,&pdf,*sz);
			       pdf=pdf+*sz;
				sz++;

			       rlen+=charger_db_S(&sh,fname_time,&pdf,*sz);
			       pdf=pdf+*sz;
				sz++;

				pcl = (char *) malloc(sizeof(long));
				copy_bytes(pdf, pcl,sizeof(long));
				pdf=pcl;
				lg =(long *) pdf;
		         rlen+=charger_db_L(&sh,pdf,sec_last_access,*lg);
				pdf=pdf+*sz;
				sz++;

				
				copy_bytes(pdf, pcl, sizeof(long));
				pdf=pcl;
				lg=(long *) pdf;	 
			      rlen+=charger_db_L(&sh,pdf,ms_last_access,*lg);
				pdf=pdf+*sz;
				sz++;

				
				copy_bytes(pdf, pcl, sizeof(long));
				pdf=pcl;
				lg=(long *) pdf;	 
			      rlen+=charger_db_L(&sh,pdf,sec_last_modif,*lg);
				pdf=pdf+*sz;
				sz++;
				
				copy_bytes(pdf, pcl, sizeof(long));
				pdf=pcl;
				lg=(long *) pdf;	 
			       rlen+=charger_db_L(&sh,pdf,ms_last_modif,*lg);
				free(pcl);  
				ih=(int *) tempo;
				*ih=rlen;
				break;
					 

			case AUR_TEXT:

				
				sz=(short *) buffAddr;
				pdf=buffAddr+sizeof(short)*temp;
				rlen=rlen+temp*2*sizeof(short);				
				i=0;
				while(i++<temp)
					{
				        rlen+=charger_db_S(&sh,i+25,&pdf,*sz);
					pdf=pdf+*sz;
					sz++;
					}
				ih=(int *) tempo;
				*ih=rlen;
				break;

			default :
				ih=(int *) tempo;
			        *ih=rlen;
	
 		}  /* switch  */
					
	*recAddr = tempo; /* the address of the record NADf */
	return 0;
				
	} else if (rec_type == AUR_TRAILER) {
		atrail=(audit_trailer_t *) buffAddr;
		nextPath = malloc(atrail->at_namelen);
		copy_bytes((char *) (atrail+1), nextPath, atrail->at_namelen);
	        rlen=sizeof(int)+6*sizeof(short);
	        sh=(short *) (rlen_ptr + sizeof(int) );
	        rlen+=charger_L(&sh,attime,atrail->at_time);
	        rlen+=charger_SH(&sh,atrecord_type,atrail->at_record_type);
	        rlen+=charger_SH(&sh,atnamelen,atrail->at_namelen);
	        ih=(int *) tempo;
	        *ih=rlen;
        	*recAddr = tempo;   /* return the address of the trail audit trail */
		return SUNTRAILER;		
	} else {
		fprintf(stderr,"fadaptor: unknown record type: %d\n", rec_type);
		exit(1);
	}
} 

/*****************************************************************************/
/* 									     */
/* VARIABLES GLOBALES:   /						     */
/* PRECONDITION:							     */
/*  		- pdf est l'adresse d'un entier 			     */
/*              - id est un entier court non signe			     */
/*              - sh est un pointeur vers l'adresse d'une zone alignee sur   */ /*                un demi-mot                                                */
/*              - soit sh0 la valeur de (*sh)				     */
/*									     */	/* POSTCONDITION: 							     */
/*   		- Soit lg = la taille en byte d'un entier;		     */	/* 		        v = la valeur pointee par pdf   		     */
/*              - alors sh0 est l'adresse de debut de trois zones contigues: */	
/*                   - la premiere de 2 bytes contenant la valeur id	     */
/*                   - la seconde de 2 bytes contenant la valeur lg	     */
/*                   - la troisieme de 'lg' bytes contenant la valeur v	     */
/*		- *sh est l'adresse du byte qui suit immediatement la 3-ieme */
/*		  zone							     */
/*              - la valeur retournee est 'lg'				     */
/* 									     */
/*****************************************************************************/ 

	
charger_db_I(sh,pdf, id, val)
short  **sh, id;
int  val;
char *pdf;
	{
	int  rlen=0;
	char *pc;
	short shaux;

	**sh=id;
	(*sh)++;
	shaux=**sh=sizeof(int);
	rlen=rlen+**sh;
	(*sh)++;
	pc= (char *) *sh;
	copy_bytes(pdf, pc, sizeof(int)); 
	*sh += sizeof(int);
	rlen += align(shaux, sh);
	return (rlen);
	}

/*****************************************************************************/
/* 									     */
/* VARIABLES GLOBALES:   /						     */
/* PRECONDITION:							     */
/*  		- val est un entier 	      				     */
/*              - id est un entier court non signe			     */
/*              - sh est un pointeur vers l'adresse d'une zone alignee sur   */ /*                un demi-mot                                                */
/*              - soit sh0 la valeur de (*sh)				     */
/*									     */	/* POSTCONDITION: 							     */
/*   		- Soit lg = la taille en byte d'un entier;		     */	
/*              - alors sh0 est l'adresse de debut de trois zones contigues: */	
/*                   - la premiere de 2 bytes contenant la valeur id	     */
/*                   - la seconde de 2 bytes contenant la valeur lg	     */
/*                   - la troisieme de 'lg' bytes contenant la valeur val    */
/*		- *sh est l'adresse du byte qui suit immediatement la 3-ieme */
/*		  zone							     */
/*              - la valeur retournee est 'lg'				     */
/* 									     */
/*****************************************************************************/ 

charger_I(sh, id, val)
short  **sh, id;
int  val;
	{
	int  rlen=0,*ih;
	int shaux;

	
	**sh=id;
	(*sh)++;
	shaux=**sh=sizeof(int);
	rlen=rlen+**sh;
	(*sh)++;
	 ih=(int *) *sh;
	*ih=val;
	ih++; 
	*sh=(short *) ih; 
	rlen += align(shaux, sh);
	return (rlen);
	}


/*****************************************************************************/
/* 									     */
/* VARIABLES GLOBALES:   /						     */
/* PRECONDITION:							     */
/*  		- val est un entier court non signe 			     */
/*              - id est un entier court non signe			     */
/*              - sh est un pointeur vers l'adresse d'une zone alignee sur   */ /*                un demi-mot       					     */
/*              - soit sh0 la valeur de (*sh)				     */
/*									     */	/* POSTCONDITION: 							     */
/*   		- Soit lg = la taille en byte d'un entier;		     */	
/*              - alors sh0 est l'adresse de debut de trois zones contigues: */	
/*                   - la premiere de 2 bytes contenant la valeur id	     */
/*                   - la seconde de 2 bytes contenant la valeur lg	     */
/*                   - la troisieme de 'lg' bytes contenant la valeur val    */
/*		- *sh est l'adresse du byte qui suit immediatement la 3-ieme */
/*		  zone							     */
/*              - la valeur retournee est 'lg'				     */
/* 									     */
/*****************************************************************************/ 


charger_UI(sh, id, val)
short  **sh, id;
unsigned int  val;
	{
	int  rlen=0; 
	unsigned int *uih;
	int shaux;

	

	**sh=id;
	(*sh)++;
	shaux=**sh=sizeof(int);
	rlen=rlen+**sh;
	(*sh)++;
	uih=(unsigned int *) *sh;
	*uih=val;
	uih++;
	*sh=(short *) uih;
	rlen += align(shaux, sh);
	return (rlen);
	}


/*****************************************************************************/
/* 									     */
/* VARIABLES GLOBALES:   /						     */
/* PRECONDITION:							     */
/*  		- pdf est l'adresse d'un entier long 			     */
/*              - id est un entier court non signe			     */
/*              - sh est un pointeur vers l'adresse d'une zone alignee sur   */ /*                un demi-mot       					     */
/*              - soit sh0 la valeur de (*sh)				     */
/*									     */	/* POSTCONDITION: 							     */
/*   		- Soit lg = la taille en byte d'un entier;		     */	
/*   		        v = la valeur pointee par pdf;	     		     */	
/*              - alors sh0 est l'adresse de debut de trois zones contigues: */	
/*                   - la premiere de 2 bytes contenant la valeur id	     */
/*                   - la seconde de 2 bytes contenant la valeur lg	     */
/*                   - la troisieme de 'lg' bytes contenant la valeur v      */
/*		- *sh est l'adresse du byte qui suit immediatement la 3-ieme */
/*		  zone							     */
/*              - la valeur retournee est 'lg'				     */
/* 									     */
/*****************************************************************************/ 

charger_db_L(sh,pdf, id, val)
short  **sh, id;
long  val;
char *pdf;
	{
	int  rlen=0; 
	long *lh;
	char *pc;
	int shaux;

	

	**sh=id;
	(*sh)++;
	shaux=**sh=sizeof(long);
	rlen=rlen+**sh;
	(*sh)++;
	pc= (char *) *sh;
	copy_bytes(pdf, pc, sizeof(long));
	*sh+=sizeof(long);
	rlen += align(shaux, sh);
	return (rlen);
	}


/*****************************************************************************/
/* 									     */
/* VARIABLES GLOBALES:   /						     */
/* PRECONDITION:							     */
/*  		- val est un entier long 				     */
/*              - id est un entier court non signe			     */
/*              - sh est un pointeur vers l'adresse d'une zone alignee sur   */ /*                un demi-mot       					     */
/*              - soit sh0 la valeur de (*sh)				     */
/*									     */	/* POSTCONDITION: 							     */
/*   		- Soit lg = la taille en byte d'un entier long;		     */	
/*              - alors sh0 est l'adresse de debut de trois zones contigues: */	
/*                   - la premiere de 2 bytes contenant la valeur id	     */
/*                   - la seconde de 2 bytes contenant la valeur lg	     */
/*                   - la troisieme de 'lg' bytes contenant la valeur val    */
/*		- *sh est l'adresse du byte qui suit immediatement la 3-ieme */
/*		  zone							     */
/*              - la valeur retournee est 'lg'				     */
/* 									     */
/*****************************************************************************/ 

charger_L(sh, id, val)
short  **sh, id;
long  val;
	{
	int  rlen=0; 
	long *lh;
	int shaux;

	

	**sh=id;
	(*sh)++;
	shaux=**sh=sizeof(long);
	rlen=rlen+**sh;
	(*sh)++;
	lh=(long *) *sh;
	*lh=val;
	lh++;
	*sh=(short *) lh;
	rlen += align(shaux, sh);
	return (rlen);
	}



/*****************************************************************************/
/* 									     */
/* VARIABLES GLOBALES:   /						     */
/* PRECONDITION:							     */
/*  		- val est un entier court 				     */
/*              - id est un entier court non signe			     */
/*              - sh est un pointeur vers l'adresse d'une zone alignee sur   */ /*                un demi-mot       					     */
/*              - soit sh0 la valeur de (*sh)				     */
/*									     */	/* POSTCONDITION: 							     */
/*   		- Soit lg = la taille en byte d'un entier court;	     */	
/*              - alors sh0 est l'adresse de debut de trois zones contigues: */	
/*                   - la premiere de 2 bytes contenant la valeur id	     */
/*                   - la seconde de 2 bytes contenant la valeur lg	     */
/*                   - la troisieme de 'lg' bytes contenant la valeur val    */
/*		- *sh est l'adresse du byte qui suit la 3-ieme zone et qui   */
/*		  soit aligne sur un demi-mot				     */
/*              - la valeur retournee est 'lg'				     */
/* 									     */
/*****************************************************************************/ 

charger_SH(sh, id, val)
short  **sh,id;
short val;
	{
	int rlen=0;
	int shaux;

	

	**sh=id;
	(*sh)++;
	shaux=**sh=sizeof(short);
	rlen=rlen+**sh;
	(*sh)++;
	**sh=val;
	(*sh)++;
	rlen += align(shaux, sh);
	return(rlen);
	}

/*****************************************************************************/
/* 									     */
/* VARIABLES GLOBALES:   /						     */
/* PRECONDITION:							     */
/*  		- val est un entier court non signe  			     */
/*              - id est un entier court non signe			     */
/*              - sh est un pointeur vers l'adresse d'une zone alignee sur   */ /*                un demi-mot       					     */
/*              - soit sh0 la valeur de (*sh)				     */
/*									     */	/* POSTCONDITION: 							     */
/*   		- Soit lg = la taille en byte d'un entier court non signe;   */	
/*              - alors sh0 est l'adresse de debut de trois zones contigues: */	
/*                   - la premiere de 2 bytes contenant la valeur id	     */
/*                   - la seconde de 2 bytes contenant la valeur lg	     */
/*                   - la troisieme de 'lg' bytes contenant la valeur val    */
/*		- *sh est l'adresse du byte qui suit la 3-ieme zone et qui   */
/*		  soit aligne sur un demi-mot				     */
/*              - la valeur retournee est 'lg'				     */
/* 									     */
/*****************************************************************************/ 

charger_USH(sh, id, val)
short  **sh,id;
unsigned short val;
	{
	int rlen=0;
 	unsigned short *ush;
	int shaux;

	

	**sh=id;
	(*sh)++;
	shaux=**sh=sizeof(unsigned short);
	rlen=rlen+**sh;
	(*sh)++;
	ush=(unsigned short *) *sh;
	*ush=val;
	ush++;
	*sh=(short *) ush;
	rlen += align(shaux, sh);
	return(rlen);
	}


/*****************************************************************************/
/* 									     */
/* VARIABLES GLOBALES:   /						     */
/* PRECONDITION:							     */
/*  		- val1 est l'adresse d'une zone de longueur val2 bytes       */
/*              - id est un entier court non signe			     */
/*              - sh est un pointeur vers l'adresse d'une zone alignee sur   */ /*                un demi-mot       					     */
/*              - soit sh0 la valeur de (*sh)				     */
/*									     */	/* POSTCONDITION: 							     */
/*              - Soit lgr le plus petit multiple de ALIGN_REC superieur ou  */
/*                egal a val2 						     */
/*              - alors sh0 est l'adresse de debut de trois zones contigues: */	
/*                   - la premiere de 2 bytes contenant la valeur id	     */
/*                   - la seconde de 2 bytes contenant la valeur lgr	     */
/*                   - la troisieme de 'lgr' bytes dont les val2 premiers    */
/*                     sont ceux pointes par val1; le reste etant des blancs */
/*		- *sh est l'adresse du byte qui suit immediatement la 3-ieme */
/*                zone	              					     */
/*              - la valeur retournee est lgr				     */
/* 									     */
/*****************************************************************************/ 

int  charger_STR(sh, id, val1,val2)
short  **sh,id;
char *val1;
short val2;
	{
	char *ch;
	int rlen=0;
	int shaux;

	**sh=id;
	(*sh)++;
	shaux=**sh=val2;
	rlen=rlen+**sh;
	(*sh)++;
	ch=(char *) *sh;
	copy_bytes(val1,ch,val2);
	ch=ch+val2;
	*sh=(short *) ch;
	rlen += align(shaux, sh);
	return(rlen);
	}

/*****************************************************************************/
/* 									     */
/* VARIABLES GLOBALES:   /						     */
/* PRECONDITION:							     */
/*  		- val est une valeur entiere 				     */
/*              - pdf est un pointeur vers l'adresse d'une zone de val bytes */
/*              - id est un entier court non signe			     */
/*              - sh est un pointeur vers l'adresse d'une zone alignee sur   */ /*                un demi-mot       					     */
/*              - soit sh0 la valeur de (*sh)				     */
/*									     */	/* POSTCONDITION: 							     */
/*              - Soit lgr le plus petit multiple de ALIGN_REC superieur ou  */
/*                egal a val 						     */
/*              - alors sh0 est l'adresse de debut de trois zones contigues: */	
/*                   - la premiere de 2 bytes contenant la valeur id	     */
/*                   - la seconde de 2 bytes contenant la valeur lgr	     */
/*                   - la troisieme de 'lgr' bytes dont les val premiers     */
/*                     sont ceux pointes par *pdf; le reste etant des blancs */
/*		- *sh est l'adresse du byte qui suit la 3-ieme zone et qui   */
/*		  soit aligne sur un demi-mot				     */
/*              - la valeur retournee est 'lgr'				     */
/* 									     */
/*****************************************************************************/ 

int  charger_db_S(sh, id, pdf,val)
short  **sh,id;
char **pdf;
short val;
	{
	char *ch;
	int rlen=0;
	int shaux;

	**sh=id;
	(*sh)++;
	shaux=**sh=val;
	rlen=rlen+(**sh);
	(*sh)++;
	ch=(char *) *sh;
	copy(*pdf,ch,val);
	ch=ch+val;
	*sh=(short *) ch;
	rlen += align(shaux, sh);
	return(rlen);
	}

/*****************************************************************************/
/* 									     */
/* VARIABLES GLOBALES:   /						     */
/* PRECONDITION:							     */
/*  		- shaux est une valeur entiere 				     */
/*              - sh est un pointeur vers l'adresse d'une zone  de bytes     */
/*              - soit sh0 la valeur de (*sh)				     */
/*									     */	/* POSTCONDITION: 							     */
/*              - Soit al = shaux*(ALIGN_REC - 1))%ALIGN_REC;   	     */
/*		- *sh pointe 'al' bytes au dela de sh0;                      */
/*		- ces bytes sont egaux au caractere blanc 		     */
/*              - la valeur retournee est al				     */
/* 									     */
/*****************************************************************************/ 

align(shaux, sh)
int shaux; 
short **sh;
{
	int i;
	char *ch;

	ch = (char *) (*sh);
	for (i=0; i<((shaux*(ALIGN_REC - 1))%ALIGN_REC); i++) { 
	/* for (i=0; i< shaux % 2; i++) {   */
		*ch= 32;
		ch++;
	}
	*sh = (short *) ch;
	return(i);
}
  
copy(from, to, n)
char *from, *to;
int n;
     {   
     while((n>0) && (*from!='\0'))
	{
	*to++ = *from++;
	n--;
	}
    while(n>0)
	{
	*to++=32;
	n--;
	}	
}

/* 	class_event()
*
*	sh 	: address of the pointer to the current position in the NADF 
*	          record area;
*	ent 	: power of 2 
*	The effect is to construct from 'ent' an external (textual) 
*	representation of  the event type, build the audit data containing this
* 	external event and write this constructed audit data at the end of
*	the NADF record buffer.
*	
*			(see audit.h)
*/ 
class_event(sh,ent)
short **sh;
unsigned int ent;
{
	extern tab_event event_class[];  /* from event.type.tab */
	int rlen = 0;
	switch (ent)
	{
	case 1 :
		rlen+=charger_STR(sh,event,event_class[1].type_name,32);
		 break;
	case 2 : rlen+=charger_STR(sh,event,event_class[2].type_name,32);
		 break;
	case 4 : rlen+=charger_STR(sh,event,event_class[3].type_name,32);
		 break;
	case 8 : rlen+=charger_STR(sh,event,event_class[4].type_name,32);
		 break;
	case 16 : rlen+=charger_STR(sh,event,event_class[5].type_name,32);
		 break;
	case 32 : rlen+=charger_STR(sh,event,event_class[6].type_name,32);
			 break;
	case 64 : rlen+=charger_STR(sh,event,event_class[7].type_name,32);
		 break;
	case 128 : rlen+=charger_STR(sh,event,event_class[8].type_name,32);
		 break;
	case 256 : rlen+=charger_STR(sh,event,event_class[9].type_name,32);
		 break;
	case 512 : rlen+=charger_STR(sh,event,event_class[10].type_name,32);
		 break;
	case 1024 : rlen+=charger_STR(sh,event,event_class[11].type_name,32);
		 break;
	case 10 : rlen+=charger_STR(sh,event,event_class[12].type_name,32);
		 break;
	case 3 : rlen+=charger_STR(sh,event,event_class[13].type_name,32);
		break;
	case 11 : rlen+=charger_STR(sh,event,event_class[14].type_name,32);
		 break;
	case 258 : rlen+=charger_STR(sh,event,event_class[15].type_name,32);
		 break;
	}
	return(rlen);
}

/*	vclose()
*	
*	- fdesc is global variable = file descriptor of the current native file 
*	- The effect is to close the current native file 
*	- Returns 0 if success of close;
*		  SUNBADCLOSE otherwise
*/

vclose()
{
    if (fdesc > -1)  /* if the file descriptor is valid */
        {                 
        free(buffAddr);   /* release the memory area */
        free(RLEN_PTR);
        close(fdesc);
	var_header = 0;
        return(0);            /* success */            
        }
    return(SUNBADCLOSE);        /* error: invalid 'fdesc' value */
    }

/*	readslow()
*	
*	try to read n bytes from 'fd' 
*	if the read bytes < n loop trying to read remaining bytes bytes
*	if an error occurs during reading return SUNBADREAD;
* 	otherwise return 0;
*
*/
static int readslow(fd, buf, len) 
int fd;
char* buf;
unsigned len;
{
    int rc, max;

    max = len;
    /* fprintf(stderr, "fd:%d len:%d\n", fd, len); */
    while (len) {
	rc = read (fd, buf, len);
	if (!isOnline) 
		return rc;
	if (rc < 0)     
		return rc;
	if (rc == len)  
		return max;
	if (rc == 0)    
		sleep(1);
	buf += rc;
	len -= rc;
    }
    return 0;
}

